dojo.provide("zero.form.widget.Form");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.Button");
dojo.require("zero.form.common.components.UidMixin");
dojo.require("zero.form.widget.FieldWrapperContainer");
dojo.require("zero.form.widget.FormController");
dojo.require("zero.resource.DataStore");

dojo.declare("zero.form.widget.Form",
             [dijit._Widget, dijit._Templated, zero.form.common.components.NlsMixin], {

		widgetsInTemplate: true,
		formModel:"",
		templatePath: dojo.moduleUrl("zero","form/widget/templates/form.html"),
		dataId:"",
		content: null,
		title: "",
		description: "",
		fields: null,
		controller: null,
		dataItem: null,
		submitLabel: "",
		clearLabel:"",
		dataGridId:"",
		formFields: "",
		store: null,
		itemId: "",
		action: "",		//Support simply URL POST like regular form
		contextRoot: "",
		
		postMixInProperties: function()
		{
			this.inherited("postMixInProperties",arguments);
			this.submitLabel = this.getString("SUBMIT");
			this.saveLabel = this.getString("SAVE");
			this.clearLabel = this.getString("CLEAR");
		},
		
		startup: function()
		{
			this.inherited("startup",arguments);
			if (!this.formModel){
				if(this.store && (this.store instanceof zero.resource.DataStore)){
					var storeMetadata = this.store.getMetadata();
					var formModel = this._convertZRMStoreToFormModel(storeMetadata);
					this.setContent(formModel);
				}
				return;
			} 
				
			this.formFields = {};
			
			var loadingFromServerErrorString = this.getString("LOADING_FORM_MODEL_FAILED",{formModelName:this.formModel});
			dojo.xhrGet({
				url:this.formModel,
				load:dojo.hitch(this, "setContent"),
				handleAs: 'json',
				error: function()
						 {
					     	console.debug(loadingFromServerErrorString);
						 }
				
			}); 
			
		},
		
		//TODO: Will remove under story 719 in next sprint
		onGridSelection: function()
		{
			var dataGrid = dijit.byId(this.dataGridId);
			var rows = dataGrid.grid.selection.getSelected();
            var item = dataGrid.grid.model.getRow(rows[0]).__dojo_data_item;
            this.setValues(item);
		},
		
		clearDataItem: function()
		{
			this.setDataItem(null);
		},
		
		setDataStore: function(/*zero.resource.DataStore */ dataStore)
		{
			if (this.controller)
			{
				this.controller.setStore(dataStore);
			}
			else
			{
				throw new Error("zero.form.widget.Form: the form doesn't have a model");
			}
		},
		
		getDataStore: function(){
			if (this.controller)
			{
				this.controller.getStore();
			}
			else
			{
				throw new Error("zero.form.widget.Form: the form doesn't have a model");
			}
		},
		
        setDataItem: function(dataItem)
        {
            this.dataItem = dataItem;
            this.btnLabel.innerHTML = (dataItem)?this.saveLabel:this.submitLabel;
			this.toggleMutabilityOfFields(dataItem);

        },

		toggleMutabilityOfFields: function(dataItem)
		{
			var field;
			for(var fieldId in this.formFields)
			{
				field = this.formFields[fieldId];
				if(field.contentJson && field.contentJson.immutable)
				{
					field.setDisabled(dataItem);
				}
			}
		},
		
		setUpdateDataItem: function(/* Item of data store */ dataItem){
			if(dataItem){
				this.setValues(dataItem);				
			}else{
				this.onError("Cannot set empty item to form.");
			}
		},

		setValues: function(/*object*/ values)
		{
			var field;
			for(var fieldId in this.formFields)
			{
				field = this.formFields[fieldId];
				if(field.contentJson && field.contentJson.dataId)
				{
					var value = values[field.contentJson.dataId];
					if(value !== null && value !== undefined) {
						field.setValue(value);
					} else {
						field.setValue("");
						console.warn("Empty field '" + field.contentJson.dataId + "' because it's undefined");
					}
				}
			}
           this.setDataItem(values);
		},

		removeContent: function()
		{
			this.wrapperContainerNode.removeAll();
		},

		getContent: function()
		{
			var content = {};
			content.model = this.dataId;
			content.title = this.getTitle();
			content.description = this.getDescription();
			this.addFieldsToContent(content);
			return content;
		},
		
		addFieldsToContent: function(content)
		{
			content.fields = [];
			var field = {};
			var tmpField;
			var fields = this.getFields();
			for(var fieldId in fields)
			{
				field = fields[fieldId].getContent();
				content.fields.push(field);
			}
			
		},
		
		getFields: function()
		{
			return this.formFields;
		},
		
		setContent: function(jsonData)
		{
			this.content = jsonData;
			this.dataId = jsonData.model;
			if (this._isStoreBased(jsonData))
			{
				var controllerParams = {resourceCollection:this.dataId};
				//Set contextRoot
				if(this.contextRoot){
					controllerParams.contextRoot = this.contextRoot;
				}
				//Backward compatibility to data grid
				//Will remove under story 719 in next sprint
				if(this.dataGridId)
				{
					var dataGrid = this._hookWithGrid(this.dataGridId);
					controllerParams.store = dataGrid.store;
				}
				//Add the support of data store, and the priority of specified data store is higher than store in data grid
				if(this.store){
					controllerParams.store = this.store;
				}
				
				this.controller = new zero.form.widget.FormController(controllerParams);
			}
			this.setTitle(jsonData.title);
			this.setDescription(jsonData.description);
			this.addFields( jsonData.fields);
			//Setup  update data item into form if update
			if(this.itemId){
				this._setupUpdateItem();
			}
		},
		
		_isStoreBased: function(jsonData){
			var isStoreBased = false;
			if(jsonData.model||this.store||(this.dataGridId&&dijit.byId(this.dataGridId).store)){
				if(this.action==null||this.action==""){
					isStoreBased =  true;
				}
			}
			return isStoreBased;
			
		},
		
		_hookWithGrid: function(dataGridId){
			var dataGrid = dijit.byId(dataGridId);
			dojo.connect(dataGrid.grid,"onSelected",this,"onGridSelection");
			dojo.connect(dataGrid,"deleteRows",this,"clearDataItem");
			dojo.connect(dataGrid,"reload",this,"clearDataItem");
			return dataGrid;
		},
		
		_setupUpdateItem: function(){
			if(this.controller.isStoreReady){
					this.controller.getItemById(this.itemId,dojo.hitch(this,this.setUpdateDataItem));	
			}else{
					//Try to fetch the update item if the store is not ready
					setTimeout(dojo.hitch(this,this._setupUpdateItem),100);
			}
		},
		
		setTitle: function(newTitle)
		{
			this.title = newTitle;
			this.refreshTitle();
		},
		
		refreshTitle: function()
		{
			this.titleNode.innerHTML = this.title;
		},
		
		setDescription: function(newDescription)
		{
			this.description = newDescription;
			this.refreshDescription();
		},
		
		refreshDescription: function()
		{
			this.descriptionNode.innerHTML =this.description ;
		},
		
		getDescription: function()
		{
			return this.description;
		},
		
		getTitle: function()
		{
			return this.title;
		},
		
		addFields: function(fields)
		{
			var field;
			this.formFields = {};
			for(var i=0, l = fields.length;i<l;  i++)
			{
				this.addField(fields[i]);
			}			
		},
		
		addField: function(fieldJson)
		{
			var wrapper = this.wrapperContainerNode.createElement( 
				{ contentJson: fieldJson,
				  title: fieldJson.title,
				  hint: fieldJson.hint,
				  type: fieldJson.type,
				  containingForm:this,
				  blockEventsInitiation:this.blockEventsInitiation});
			this.formFields[fieldJson.shortName] = wrapper.getWidget();			
		},
	
		
		getValuesForDB: function()
		{
			var values = {};
			var field;
			for(var fieldId in this.formFields)
			{
				field = this.formFields[fieldId];
				if(field.contentJson && field.contentJson.dataId)
				{
					var fieldValue = field.getValueForDB();
					// use 0 instead NaN value since we use default dijit.getValue()
					if(isNaN(fieldValue) && typeof(fieldValue) == "number") {
						console.warn("Field '" + field.contentJson.dataId + "' is NaN, use 0 as default value");
						values[field.contentJson.dataId] = 0;
					} else {
						values[field.contentJson.dataId] = fieldValue;
					}
				}
			}
			return values;
		},
		
		_convertZRMStoreToFormModel: function(storeMetadata){
			 var formMap = {};
			 formMap.title = "Title";
        	formMap.description = "Description";
        	formMap.model = storeMetadata.type;
        	formMap.fields = [];        
        	// iterate over fileds
	        var formField = [];
	        for(var fieldId in storeMetadata.fields){
	        	if(fieldId==="id"||fieldId==="updated") continue;
	        	var fieldAttMap = storeMetadata.fields[fieldId];
	            var formField = {};
	            formField.dataId = fieldId;
	            formField.title =  !fieldAttMap.label ? fieldAttMap.label : fieldId;
	            formField.shortName = fieldId+ "_id";
	            formField.required =  (fieldAttMap.required != null ? fieldAttMap.required : false);  
	            formField.type = this._getFormType(fieldAttMap.type);  
	            formField.description =  (!fieldAttMap.description) ? fieldAttMap.description : ""; 
	            formField.default_value =  (!fieldAttMap.default_value) ? fieldAttMap.default_value : "";  
	            formField.immutable = false;
	            formMap.fields.push(formField);
	            //TODO: Should support options and decimal
	        }
	        return formMap;
		},
		
		 MODEL_FORM_TYPES : {
	        "string":"Text",
	        "decimal":"Number",
	        "integer":"Number",
	        "float":"Number",
	        "boolean":"Checkbox",
	        "date":"Date",
	        "time":"Time",
	        "date-time":"Time"
		 },
		
		_getFormType: function(modelType){
			return this.MODEL_FORM_TYPES[modelType];
		},
		
		submitPressed: function()
		{
			this.doSubmit();
			this.onSubmitted();
		},
		
		doSubmit: function()
		{
			var _self = this;
			//Simply POST first
			if(this.action&&""!=this.action){
				this._postFormToURL(this.getValuesForDB(),this.action);
			}else if (this.controller){
				if(this.dataItem)
				{
					//Update
					this.controller.updateItem(this.dataItem,this.getValuesForDB(),this.onUpdateComplete, this.onUpdateError);
					this.onSubmitReturn(this.dataItem);
				}
				else
				{
					//Create 
					//Now, let the form update this data item after created
					var dataItem = this.controller.addItem(this.getValuesForDB());
					//Only works for REST store, the ZRM data store cannot return the saved data item
					this.setUpdateDataItem(dataItem);
					this.onSubmitReturn(dataItem);
				}
			}
		},
		
		_postFormToURL: function(values,actionURL){
			var _self = this;
			var err;
			var args = {
				url: actionURL,
				postData: dojo.toJson(values),
			    contentType: "application/json",
	            load: function(resp, ioArgs) {
	            	_self.onSubmitReturn(resp);
	            },
	            error: function(resp,ioArgs){
	            	err = new Error(ioArgs.xhr.status + ": " + ioArgs.xhr.statusText);
	                err.responseText = ioArgs.xhr.responseText;
	                _self.onSubmitError(err);
	            }
	            
			};
			var deferred = dojo.rawXhrPost(args);
		},
		
		clearPressed: function()
		{
			var field;
			for(var fieldId in this.formFields)
			{
				this.formFields[fieldId].setValue("");
			}
			this.setDataItem(null);
			this.onCleared();
		},
		
		onSubmitted: function(){
			// No need to do anything. This method is here just so that client code
       		// can connect observers to it.
		},
		
		onCleared: function(){
			// No need to do anything. This method is here just so that client code
       		// can connect observers to it.
		},
		
		onError: function(e){
			console.warn("Error: " + e);
			//Callback if some error happens
			// No need to do anything. This method is here just so that client code
       		// can connect observers to it.
		},
		
		onLoadDataItemError: function(e){
			console.warn("Error when loading data item." + e);
			// No need to do anything. This method is here just so that client code
       		// can connect observers to it.
		},
		
		onSubmitReturn: function(returnValues){
			// No need to do anything. This method is here just so that client code
       		// can connect observers to it.
       		//summary: Callback after the submit data return
		},
		
		onSubmitError: function(){
			
		},
		
		onUpdateComplete:function(dataItem){
			//Callback function when update complete	
		},
		
		onUpdateError: function(error){
			//Callback function when update failed
			console.warn("Update fail." + error);
		}
});
